# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-22 12:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0006_auto_20171110_1501'),
    ]

    operations = [
        migrations.AddField(
            model_name='myuser',
            name='access_token',
            field=models.TextField(blank=True, null=True, verbose_name='OAuth2 Access Token'),
        ),
    ]
